#!/usr/bin/env python
# encoding: utf-8

'''
Date: 25.05.2012
Author: Attraktor, Hacker- & Makerspace Hamburg/Germany. Florian florian@mex21.net
See also: http://wiki.attraktor.org/CNC-Fr%C3%A4se_%28Technik%29

This scipt uses GladeVCP. It is available in LinuxCNC v2.5+

Dependencies:
	- curl
	- wmctrl
	- imagemagick (import)
	- python-setuptools
		> sudo apt-get install curl wmctrl imagemagick python-setuptools

	- tweepy:
		"Download as ZIP" from https://github.com/tweepy/tweepy/downloads
		extract
		> sudo python ./setup.py install

	(installed by default)
	- grep
	- tail
	- sed
	- ps
	- lsof
	- tr
'''

import hal
import glib

from datetime import datetime
import time
import shutil
import urllib
import os
import tweepy

#TODO: Twitter-OAuth und Twitpic keys hier einsetzen. Die befinden sich beim Vorstand und bei Florian

SCREENSHOT_PATH = "/home/cnc/linuxcnc/twitter/Dropbox/Public/" # path to save public files in, auto-syncing dropbox folder

debug = 1

class HandlerClass:
	'''
	class with gladevcp callback handlers
	'''

	def on_led_isrunning_hal_pin_changed(self,widget,data=None):
		try:
			# Verify program is running
			if os.popen('/usr/bin/halcmd getp halui.program.is-running').readline().strip() == "TRUE":
				print "tweeting..."

				t = datetime.now()

				# Get PID of AXIS
				axispid = int(os.popen('ps -C axis -o pid=').read())
				if debug:
					print "axispid: %s" % axispid

				# Get ngc-filename from window title
				ngcfilename = os.popen('wmctrl -l | grep AXIS | tail -n1 | sed "s/^.*N\/A\ //g" | sed "s/\ -\ AXIS [0-9]*\.[0-9]*\.[0-9]* on Replicator$//" | tr -d "\\n"').read()
				if debug:
					print "ngcfilename: %s" % ngcfilename

				# Get ngc-filepath from open file pointer list
				ngcfilepath = os.popen('lsof -F n -p%s | grep "%s" | tail -n1 | sed "s/^n//g" | tr -d "\\n"' % (axispid, ngcfilename)).read()
				if debug:
					print "ngcfilepath: %s" % ngcfilepath

				uploadfilename = "%s_%s" % (t.strftime("%Y-%m-%d_%H-%M-%S"), ngcfilename)
				if debug:
					print "uploadfilename: %s" % uploadfilename

				uploadfilepath = os.path.join(SCREENSHOT_PATH, uploadfilename)
				if debug:
					print "uploadfilepath: %s" % uploadfilepath

				shorturl = ""

				if self.tweet_gcode == 1:
					# Copy ngc file to auto-syncing dropbox folder
					shutil.copy(ngcfilepath, uploadfilepath)

					# Get Dropbox public URL
					ngcpuburl = os.popen('dropbox puburl "%s" | tr -d "\\n"' % uploadfilepath).read()
					if debug:
						print "ngcpuburl: %s" % ngcpuburl

					# shorten URL
					shorturl = " G-Code: %s" % os.popen('curl -s "http://is.gd/create.php?format=simple&url=%s"' % urllib.quote(ngcpuburl)).readline().strip()
					if debug:
						print "shorturl: %s" % shorturl

				# Concatenate twitter message
				message = ("%s: %s wird gefräst.%s" % (t.strftime("%d.%m.%Y, %H:%M:%S"), ngcfilename[:40], shorturl))[:140]
				if debug:
					print "message: %s" % message

				if self.tweet_screenshot == 0:
					# tweet directly
					if debug:
						print "tweeting with tweepy"
					auth = tweepy.OAuthHandler(TWITTER_CONSUMER_TOKEN, TWITTER_CONSUMER_SECRET)
					auth.set_access_token(TWITTER_OAUTH_TOKEN, TWITTER_OAUTH_SECRET)
					api = tweepy.API(auth)
					api.update_status(message)
				else:
					# Concatenate screenshot file path
					screenshotfilepath = "%s.png" % uploadfilepath.replace(",", "_").replace(";", "_")
					if debug:
						print "screenshotfilepath: %s" % screenshotfilepath

					# Get window id of AXIS
					axiswinid = os.popen('wmctrl -l | grep AXIS | tail -n1 | sed "s/\ .*//g" | tr -d "\\n"').read()
					if debug:
						print "axiswinid: %s" % axiswinid

					# Take a screenshot, save it into auto-syncing dropbox folder
					if debug:
						print "taking screenshot"
					os.popen("import -window %s -quality 80 '%s'" % (axiswinid, screenshotfilepath)).read()

					if debug:
						print "uploading to twitpic"
					# Upload Screenshot to twitpic
					os.popen('curl -v -F "consumer_token=%s" -F "consumer_secret=%s" -F "oauth_token=%s" -F "oauth_secret=%s" -F "message=%s" -F "key=%s" -F "media=@%s" http://api.twitpic.com/1/uploadAndPost.json' % (TWITTER_CONSUMER_TOKEN, TWITTER_CONSUMER_SECRET, TWITTER_OAUTH_TOKEN, TWITTER_OAUTH_SECRET, message, TWITPIC_KEY, screenshotfilepath))

				print "tweeting done"
		except:
			print "error"

	# Event handler
	def on_btn_tweet_screenshot_clicked(self,widget,data=None):
		if self.tweet_screenshot == 0:
			self.btn_tweet_screenshot.set_image(self.img_screenshot)
			self.tweet_screenshot = 1
			print "screenshot upload enabled"
		else:
			self.btn_tweet_screenshot.set_image(self.img_noscreenshot)
			self.tweet_screenshot = 0
			print "screenshot upload disabled"

	# Event handler
	def on_btn_tweet_gcode_clicked(self,widget,data=None):
		if self.tweet_gcode == 0:
			self.btn_tweet_gcode.set_image(self.img_gcode)
			self.tweet_gcode = 1
			if debug:
				print "gcode upload enabled"
		else:
			self.btn_tweet_gcode.set_image(self.img_nogcode)
			self.tweet_gcode = 0
			if debug:
				print "gcode upload disabled"

	def __init__(self, halcomp,builder,useropts):
		'''
		Handler classes are instantiated in the following state:
		- the widget tree is created, but not yet realized (no toplevel window.show() executed yet)
		- the halcomp HAL component is set up and the widhget tree's HAL pins have already been added to it
		- it is safe to add more hal pins because halcomp.ready() has not yet been called at this point.

		after all handlers are instantiated in command line and get_handlers() order, callbacks will be
		connected with connect_signals()/signal_autoconnect()

		The builder may be either of libglade or GtkBuilder type depending on the glade file format.
		'''

		self.halcomp = halcomp
		self.builder = builder


		self.tweet_gcode = 1
		self.img_gcode = self.builder.get_object('img_gcode')
		self.img_nogcode = self.builder.get_object('img_nogcode')
		self.btn_tweet_gcode = self.builder.get_object('btn_tweet_gcode')

		self.tweet_screenshot = 1
		self.img_screenshot = self.builder.get_object('img_screenshot')
		self.img_noscreenshot = self.builder.get_object('img_noscreenshot')
		self.btn_tweet_screenshot = self.builder.get_object('btn_tweet_screenshot')

		#self.builder.get_object('tbl_twittersettings').set_sensitive(True)
		# doesn't work. see custom_postgui.hal and twitter_unlock_gui.py


def get_handlers(halcomp,builder,useropts):
	'''
	this function is called by gladevcp at import time (when this module is passed with '-u <modname>.py')

	return a list of object instances whose methods should be connected as callback handlers
	any method whose name does not begin with an underscore ('_') is a  callback candidate

	the 'get_handlers' name is reserved - gladevcp expects it, so do not change
	'''
	return [HandlerClass(halcomp,builder,useropts)]

